import simulation.event.*;
import simset.*;
import random.*;

public class CarWashSimulation extends Simulation {
    int noOfCarWashers;
    double simPeriod = 200; 
    Head tearoom = new Head();
    Head waitingLine = new Head();
    Random random = new Random(5);
    int noOfCustomers, maxLength;
    double throughTime;
    long startTime = System.currentTimeMillis();

    CarWashSimulation(int n) { 
        noOfCarWashers = n; 
        for (int i = 1; i <= noOfCarWashers; i++)
            new CarWasher().into(tearoom);
        new CarArrival().schedule(0);
        runSimulation(simPeriod + 1000000);
        report();
    }
    
    void report() {
        System.out.println(noOfCarWashers + " car washer simulation");
        System.out.println("No.of cars through the system = " + 
                           noOfCustomers);
        java.text.NumberFormat fmt = java.text.NumberFormat.getNumberInstance();
        fmt.setMaximumFractionDigits(2);
        System.out.println("Av.elapsed time = " + 
                           fmt.format(throughTime/noOfCustomers));
        System.out.println("Maximum queue length = " + maxLength);
        System.out.println("\nExecution time: " +
                           fmt.format((System.currentTimeMillis() 
                                       - startTime)/1000.0) +			   
                           " secs.\n");
    }
	
    class CarWasher extends Link {}

    class Car extends Link { double entryTime = time(); }

    class CarArrival extends Event {		
        public final void actions() {
            if (time() <= simPeriod) {
	            Car theCar = new Car();
	            theCar.into(waitingLine);
	            int qLength = waitingLine.cardinal();
	            if (maxLength < qLength) 
	                maxLength = qLength;
	            if (!tearoom.empty()) 
	                new StartCarWashing().schedule(time());
	            new CarArrival().schedule(time() + random.negexp(1/11.0));
	        }
        }	
    }

    class StartCarWashing extends Event {
        public void actions() {
            CarWasher theCarWasher = (CarWasher) tearoom.first();
            theCarWasher.out();
            Car theCar = (Car) waitingLine.first();
            theCar.out();
            new StopCarWashing(theCarWasher, theCar).schedule(time() + 10);
        }
    }
		
    class StopCarWashing extends Event {
        CarWasher theCarWasher;
        Car theCar;
		
        StopCarWashing(CarWasher cw, Car c) { theCarWasher = cw; theCar = c; }
		
        public void actions() {
            theCarWasher.into(tearoom);
            if (!waitingLine.empty()) 
                new StartCarWashing().schedule(time());
            noOfCustomers++;
            throughTime += time() - theCar.entryTime;
        }
    }
	
    public static void main(String args[]) {
        new CarWashSimulation(1);
        new CarWashSimulation(2);
    } 
}