/*
  File: TimeEvent.java

  Originally written by Keld Helsgaun and released into the public domain.
  This may be used for any purposes whatsoever without acknowledgment.

  History:
  Date       Who                What
  5May2000   kh         Created public version
*/

package simulation.events;

/**
* This class may used to describe a state-event, i.e., an event 
* scheduled to occur at a specified point in time.
* <p>
* Time events are created as instances of <tt>TimeEvent</tt>-
* derived classes that override the abstract methods <tt>actions</tt>.
* <p>
* The <tt>actions</tt> method is used to specify the actions of
* of a class of time events.
*
* @see simulation.events.Event
* @see simulation.events.StateEvent
* @see simulation.events.Simulation  
*/

public abstract class TimeEvent extends Event {
    /**
    * The event time of this time event.
    */
    double eventTime;
    
    /**
    * Schedules this time event to occur at the specified event time.
    * <p>
    * The event is scheduled to occur after other time events with
    * the same event time.
    */	
    public final void schedule(final double evTime) {
        if (evTime < time)
            throw new RuntimeException("attempt to schedule event in the past");
        cancel();
        eventTime = evTime;
        Event ev = SQS.pred; 
        while (((TimeEvent) ev).eventTime > eventTime)
            ev = ev.pred;
        pred = ev; 
        suc = ev.suc;
        ev.suc = suc.pred = this;
    }
}
